@@ -1,16 +1,14 @@
 #undef __WORDSIZE
 #define __WORDSIZE 64
 
-typedef struct user_fpregs_struct
-{
+typedef struct user_fpregs_struct {
 	uint16_t cwd, swd, ftw, fop;
 	uint64_t rip, rdp;
 	uint32_t mxcsr, mxcr_mask;
 	uint32_t st_space[32], xmm_space[64], padding[24];
 } elf_fpregset_t;
 
-struct user_regs_struct
-{
+struct user_regs_struct {
 	unsigned long r15, r14, r13, r12, rbp, rbx, r11, r10, r9, r8;
 	unsigned long rax, rcx, rdx, rsi, rdi, orig_rax, rip;
 	unsigned long cs, eflags, rsp, ss, fs_base, gs_base, ds, es, fs, gs;
@@ -18,8 +16,7 @@ struct user_regs_struct
 #define ELF_NGREG 27
 typedef unsigned long long elf_greg_t, elf_gregset_t[ELF_NGREG];
 
-struct user
-{
+struct user {
 	struct user_regs_struct		regs;
 	int				u_fpvalid;
 	struct user_fpregs_struct	i387;
