@@ -1,5 +1,4 @@
-struct ipc_perm
-{
+struct ipc_perm {
 	key_t __ipc_perm_key;
 	uid_t uid;
 	gid_t gid;
